import { spawn, ChildProcess } from 'child_process';
import { DevData, DevModel } from '../route/dev/model.ts';
import { fileIsExist } from '@kevisual/use-config';
import { transformToMJS } from './build/convert.ts';

/**
 * 创建代码路径
 * @param dev
 */
export const createCodePath = async (dev: DevModel) => {
  const data = dev.data;
  let { type, code } = data;
  if (type === 'script' && code) {
    const res = await transformToMJS({ inputCode: code, id: dev.id });
    if (res.code !== 200) {
      data.codePath = '';
      data.codeStatus = 'error';
      await dev.update({ data }, { logging: false, fields: ['data'] });
      throw new Error('transformToMJS error:' + res.message);
    }
    data.codePath = res.outputPath;
    data.codeStatus = 'success';
    data.status = 'active';
    await DevModel.update({ data: { ...data } }, { where: { id: dev.id }, logging: false });
  }
};
type Dev = {
  id: string;
  key: string;
  pid: number; // 进程id
  title: string;
  description: string;
} & DevData;
// 检察app数据
export const checkAppData = async (dev: DevModel) => {
  const data = dev.data;
  let { type, codePath, cwd } = data;
  if (type === 'script') {
    let checkFileIsExist = false;
    if (codePath) {
      checkFileIsExist = fileIsExist(codePath);
    }
    if (!codePath || !checkFileIsExist) {
      await createCodePath(dev);
    }
    if (!cwd) {
      throw new Error('cwd is required');
    }
  } else if (type === 'shell') {
    if (!cwd) {
      throw new Error('cwd is required');
    }
    const { command, path } = data;
    if (!command) {
      throw new Error('command is required');
    }
    if (!path) {
      throw new Error('path is required');
    }
  }
};
export const runApp = async (dev: Dev) => {
  let { type, command, env, cwd, codePath, path } = dev;
  let pid = 0;
  if (type === 'script') {
    const scriptCommand = command || 'node';
    const params = [];
    if (scriptCommand === 'vite' || scriptCommand === 'rollup') {
      params.push('-c', codePath);
    }
    // node启动
    const childProcess = spawn(command, params, {
      env: {
        ...process.env,
        ...env,
      },
      shell: true,
      stdio: 'inherit',
      cwd,
    });
    childProcess.stdout?.on('data', (data) => {
      console.log(dev.id, 'stdout', data.toString());
    });
    childProcess.stderr?.on('data', (data) => {
      console.log(dev.id, 'stderr', data.toString());
    });
    console.log('child pid ', childProcess.pid);
    pid = childProcess.pid;
  } else if (type === 'shell') {
    // shell启动
    const childProcess = spawn(command, [path], {
      env: {
        ...process.env,
        ...env,
      },
      stdio: 'inherit',
      shell: true,
      cwd,
    });
    pid = childProcess.pid;
  }
  dev.pid = pid;
};
/**
 * check pid is exist
 * @param pid
 * @returns
 */
export const checkPid = (pid: number) => {
  try {
    process.kill(pid, 0);
    return true;
  } catch (e) {
    return false;
  }
};
export const stopApp = async (dev: Dev) => {
  const { pid } = dev;
  if (pid) {
    if (checkPid(pid)) {
      console.log('stopApp', dev.id, 'pid', pid);
      process.kill(pid);
    }
  }
};
export class DevManater {
  devList: Dev[] = [];
  async create(dev: DevModel) {
    const data = dev.data;
    await checkAppData(dev);
    let hasDev = this.devList.find((item) => item.id === dev.id || item.key === dev.key);
    if (hasDev) {
      await stopApp(hasDev);
      this.devList = this.devList.filter((item) => item.id !== dev.id && item.key !== dev.key);
    }
    const _dev = {
      id: dev.id,
      key: dev.key,
      title: dev.title,
      description: dev.description,
      pid: 0,
      ...data,
    };
    await runApp(_dev);
    this.devList.push(_dev);
    return _dev;
  }
  async stop(id: string) {
    const dev = this.devList.find((item) => item.id === id);
    if (dev) {
      await stopApp(dev);
      this.devList = this.devList.filter((item) => item.id !== id);
    }
    return dev;
  }
  async getList() {
    return this.devList;
  }
  async runScript(loadScript: string, dev: DevModel) {
    const data = dev.data;
    const { cwd } = data;
    const childProcess = spawn(loadScript, [], {
      stdio: 'inherit',
      shell: true,
      cwd: cwd,
      detached: true,
      env: {
        ...process.env,
        ...data.env,
      },
    });
    return childProcess;
  }
}

export const runScript = (dev: DevModel) => {
  const data = dev.data;
  const { type, code } = data;
  //
};
