import { app, devManager } from '@/app.ts';
import { DevModel } from './model.ts';
import { transformToMJS } from '@/modules/build/index.ts';

app
  .route({
    path: 'dev-app',
    key: 'createConfig',
    description: '创建开发的配置项，比如vite的执行配置。',
  })
  .define(async (ctx) => {
    const id = ctx.query.id;
    if (!id) {
      ctx.throw(400, 'id is required');
      return;
    }
    const dev = await DevModel.findByPk(id);
    if (!dev) {
      ctx.throw(404, 'dev not found');
      return;
    }
    const data = dev.data;
    if (!data) {
      ctx.throw(400, 'data is required');
      return;
    }
    const { type, path, code, codePath, command, env } = data;
    if (type !== 'script') {
      ctx.throw(400, 'type is not script');
      return;
    }
    const res = await transformToMJS({ inputCode: code, id: id });
    if (res.code !== 200) {
      ctx.throw(res.code, res.message);
      return;
    }
    await dev.update(
      {
        data: {
          ...data,
          codePath: res.outputPath,
          codeStatus: 'success',
        },
      },
      {
        logging: false,
        fields: ['data'],
      },
    );
    ctx.body = dev;
  })
  .addTo(app);

app
  .route({
    path: 'dev-app',
    key: 'runningList',
  })
  .define(async (ctx) => {
    const list = await devManager.getList();
    ctx.body = list;
  })
  .addTo(app);

app
  .route({
    path: 'dev-app',
    key: 'run',
  })
  .define(async (ctx) => {
    const id = ctx.query.id;
    const key = ctx.query.appKey;
    if (!id && !key) {
      ctx.throw(400, 'id or appKey is required');
      return;
    }
    let dev: DevModel;
    if (id) {
      dev = await DevModel.findByPk(id, {
        logging: false,
      });
    }
    if (!dev) {
      dev = await DevModel.findOne({
        where: {
          key,
        },
        logging: false,
      });
    }
    if (!dev) {
      ctx.throw(404, 'dev not found');
      return;
    }

    const devInfo = await devManager.create(dev);
    ctx.body = devInfo;
  })
  .addTo(app);

app
  .route({
    path: 'dev-app',
    key: 'stop',
  })
  .define(async (ctx) => {
    const id = ctx.query.id;
    if (!id) {
      ctx.throw(400, 'id is required');
      return;
    }
    const dev = await devManager.stop(id);
    ctx.body = dev;
  })
  .addTo(app);

app
  .route({
    path: 'dev-app',
    key: 'runScript',
  })
  .define(async (ctx) => {
    const id = ctx.query.id;
    const script = ctx.query.script;
    if (!id) {
      ctx.throw(400, 'id is required');
      return;
    }
    if (!script) {
      ctx.throw(400, 'script is required');
      return;
    }
    const dev = await DevModel.findByPk(id);
    if (!dev) {
      ctx.throw(404, 'dev not found');
      return;
    }
    const scripts = dev.data.scripts || {};
    const loadScript = scripts[script];
    if (!loadScript) {
      ctx.throw(404, 'script not found');
      return;
    }
    const result = await devManager.runScript(loadScript, dev);
    ctx.body = result;
  })
  .addTo(app);
